@@ -88,7 +88,7 @@ out:
  *	u32  - spec has meta characters
  *	u32  - The specs prefix_len if >= SELINUX_COMPILED_FCONTEXT_PREFIX_LEN
  *	u32  - data length of the pcre regex
- *	char - a bufer holding the raw pcre regex info
+ *	char - a buffer holding the raw pcre regex info
  *	u32  - data length of the pcre regex study daya
  *	char - a buffer holding the raw pcre regex study data
  */
@@ -266,7 +266,7 @@ static void free_specs(struct saved_data *data)
 	memset(data, 0, sizeof(*data));
 }
 
-static void usage(const char *progname)
+static __attribute__ ((__noreturn__)) void usage(const char *progname)
 {
 	fprintf(stderr,
 	    "usage: %s [-o out_file] [-p policy_file] fc_file\n"
